@@ -65,24 +65,27 @@ class ObjectComboBox(EditableComboBox):
             return True
         if qstr is None:
             qstr = self.currentText()
-        if not re.search('^[a-zA-Z0-9_\.]*$', str(qstr), 0):
+        if not re.search(r'^[a-zA-Z0-9_\.]*$', str(qstr), 0):
             return False
         objtxt = to_text_string(qstr)
+        shell_is_defined = False
         if self.help.get_option('automatic_import'):
             shell = self.help.internal_shell
             if shell is not None:
-                return shell.is_defined(objtxt, force_import=True)
-        shell = self.help.get_shell()
-        if shell is not None:
-            try:
-                return shell.is_defined(objtxt)
-            except socket.error:
-                shell = self.help.get_shell()
+                shell_is_defined = shell.is_defined(objtxt, force_import=True)
+        if not shell_is_defined:
+            shell = self.help.get_shell()
+            if shell is not None:
                 try:
-                    return shell.is_defined(objtxt)
+                    shell_is_defined = shell.is_defined(objtxt)
                 except socket.error:
-                    # Well... too bad!
-                    pass
+                    shell = self.help.get_shell()
+                    try:
+                        shell_is_defined = shell.is_defined(objtxt)
+                    except socket.error:
+                        # Well... too bad!
+                        pass
+        return shell_is_defined
 
     def validate_current_text(self):
         self.validate(self.currentText())
@@ -91,7 +94,7 @@ class ObjectComboBox(EditableComboBox):
         """Reimplemented to avoid formatting actions"""
         valid = self.is_valid(qstr)
         if self.hasFocus() and valid is not None:
-            if editing:
+            if editing and not valid:
                 # Combo box text is being modified: invalidate the entry
                 self.show_tip(self.tips[valid])
                 self.valid.emit(False, False)
@@ -254,7 +257,7 @@ class SphinxThread(QThread):
     doc : str or dict
         A string containing a raw rst text or a dict containing
         the doc string components to be rendered.
-        See spyder.utils.dochelpers.getdoc for description.
+        See spyder_kernels.utils.dochelpers.getdoc for description.
     context : dict
         A dict containing the substitution variables for the
         layout template
@@ -299,10 +302,12 @@ class SphinxThread(QThread):
                                                math=self.math_option,
                                                img_path=self.img_path)
                     html_text = sphinxify(doc['docstring'], context)
-                    if doc['docstring'] == '' and \
-                      any([doc['name'], doc['argspec'], doc['note']]):
-                        msg = _("No further documentation available")
-                        html_text += '<div class="hr"></div>'
+                    if doc['docstring'] == '':
+                        if any([doc['name'], doc['argspec'], doc['note']]):
+                            msg = _("No further documentation available")
+                            html_text += '<div class="hr"></div>'
+                        else:
+                            msg = _("No documentation available")
                         html_text += '<div id="doc-warning">%s</div>' % msg
                 except Exception as error:
                     self.error_msg.emit(to_text_string(error))
@@ -328,13 +333,14 @@ class Help(SpyderPluginWidget):
     # Signals
     focus_changed = Signal()
 
-    def __init__(self, parent):
+    def __init__(self, parent=None):
         if PYQT5:
             SpyderPluginWidget.__init__(self, parent, main = parent)
         else:
             SpyderPluginWidget.__init__(self, parent)
 
         self.internal_shell = None
+        self.console = None
 
         # Initialize plugin
         self.initialize_plugin()
@@ -394,7 +400,7 @@ class Help(SpyderPluginWidget):
         self.combo.setMaxCount(self.get_option('max_history_entries'))
         self.combo.addItems( self.load_history() )
         self.combo.setItemText(0, '')
-        self.combo.valid.connect(lambda valid: self.force_refresh())
+        self.combo.valid.connect(self.force_refresh)
 
         # Plain text docstring option
         self.docstring = True
@@ -499,7 +505,9 @@ class Help(SpyderPluginWidget):
         self.focus_changed.connect(self.main.plugin_focus_changed)
         self.main.add_dockwidget(self)
         self.main.console.set_help(self)
+
         self.internal_shell = self.main.console.shell
+        self.console = self.main.console
 
     def closing_plugin(self, cancelable=False):
         """Perform actions before parent main window is closed"""
@@ -715,21 +723,21 @@ class Help(SpyderPluginWidget):
 
     def show_rich_text(self, text, collapse=False, img_path=''):
         """Show text in rich mode"""
-        self.visibility_changed(True)
-        self.raise_()
+        self.switch_to_plugin()
         self.switch_to_rich_text()
         context = generate_context(collapse=collapse, img_path=img_path)
         self.render_sphinx_doc(text, context)
 
     def show_plain_text(self, text):
         """Show text in plain mode"""
-        self.visibility_changed(True)
-        self.raise_()
+        self.switch_to_plugin()
         self.switch_to_plain_text()
         self.set_plain_text(text, is_code=False)
 
     @Slot()
     def show_tutorial(self):
+        """Show the Spyder tutorial in the Help plugin, opening it if needed"""
+        self.switch_to_plugin()
         tutorial_path = get_module_source_path('spyder.utils.help')
         tutorial = osp.join(tutorial_path, 'tutorial.rst')
         text = open(tutorial).read()
@@ -744,12 +752,16 @@ class Help(SpyderPluginWidget):
         else:
             self.rich_text.webview.load(QUrl(url))
 
-    #------ Public API ---------------------------------------------------------
-    def force_refresh(self):
-        if self.source_is_console():
-            self.set_object_text(None, force_refresh=True)
-        elif self._last_editor_doc is not None:
-            self.set_editor_doc(self._last_editor_doc, force_refresh=True)
+    # ------ Public API -------------------------------------------------------
+    @Slot()
+    @Slot(bool)
+    @Slot(bool, bool)
+    def force_refresh(self, valid=True, editing=True):
+        if valid:
+            if self.source_is_console():
+                self.set_object_text(None, force_refresh=True)
+            elif self._last_editor_doc is not None:
+                self.set_editor_doc(self._last_editor_doc, force_refresh=True)
 
     def set_object_text(self, text, force_refresh=False, ignore_unknown=False):
         """Set object analyzed by Help"""
@@ -803,15 +815,14 @@ class Help(SpyderPluginWidget):
         index = self.source_combo.currentIndex()
         if hasattr(self.main, 'tabifiedDockWidgets'):
             # 'QMainWindow.tabifiedDockWidgets' was introduced in PyQt 4.5
-            if self.dockwidget and (force or self.dockwidget.isVisible()) \
-               and not self.ismaximized \
-               and (force or text != self._last_texts[index]):
+            if (self.dockwidget and (force or self.dockwidget.isVisible()) and
+                    not self.ismaximized and
+                    (force or text != self._last_texts[index])):
                 dockwidgets = self.main.tabifiedDockWidgets(self.dockwidget)
-                if self.main.console.dockwidget not in dockwidgets and \
-                   (hasattr(self.main, 'ipyconsole') and \
-                    self.main.ipyconsole.dockwidget not in dockwidgets):
-                    self.dockwidget.show()
-                    self.dockwidget.raise_()
+                if (self.console.dockwidget not in dockwidgets and
+                        self.main.ipyconsole is not None and
+                        self.main.ipyconsole.dockwidget not in dockwidgets):
+                    self.switch_to_plugin()
         self._last_texts[index] = text
 
     def load_history(self, obj=None):
@@ -825,9 +836,15 @@ class Help(SpyderPluginWidget):
 
     def save_history(self):
         """Save history to a text file in user home directory"""
-        open(self.LOG_PATH, 'w').write("\n".join( \
-                [to_text_string(self.combo.itemText(index))
-                 for index in range(self.combo.count())] ))
+        # Don't fail when saving search history to disk
+        # See issues 8878 and 6864
+        try:
+            search_history = [to_text_string(self.combo.itemText(index))
+                              for index in range(self.combo.count())]
+            search_history = '\n'.join(search_history)
+            open(self.LOG_PATH, 'w').write(search_history)
+        except (UnicodeEncodeError, UnicodeDecodeError, EnvironmentError):
+            pass
 
     @Slot(bool)
     def toggle_plain_text(self, checked):
@@ -887,7 +904,9 @@ class Help(SpyderPluginWidget):
         Return shell which is currently bound to Help,
         or another running shell if it has been terminated
         """
-        if not hasattr(self.shell, 'get_doc') or not self.shell.is_running():
+        if (not hasattr(self.shell, 'get_doc') or
+                (hasattr(self.shell, 'is_running') and
+                 not self.shell.is_running())):
             self.shell = None
             if self.main.ipyconsole is not None:
                 shell = self.main.ipyconsole.get_current_shellwidget()
@@ -900,8 +919,11 @@ class Help(SpyderPluginWidget):
     def render_sphinx_doc(self, doc, context=None):
         """Transform doc string dictionary to HTML and show it"""
         # Math rendering option could have changed
-        fname = self.parent().parent().editor.get_current_filename()
-        dname = osp.dirname(fname)
+        if self.main.editor is not None:
+            fname = self.main.editor.get_current_filename()
+            dname = osp.dirname(fname)
+        else:
+            dname = ''
         self._sphinx_thread.render(doc, context, self.get_option('math'),
                                    dname)
 
